"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onMessage = exports.getMessages = exports.disconnect = exports.isConnected = exports.connect = void 0;
const assert = require("assert");
const pm2 = require("pm2");
const handlers = {};
const myName = process.env.name;
const myPmId = Number(process.env.pm_id);
let myBus;
process.on('message', ({ topic, data: { targetInstanceId, data } }) => __awaiter(void 0, void 0, void 0, function* () {
    if (typeof handlers[topic] === 'function' && process.send) {
        const response = {
            type: `process:${targetInstanceId}`,
            data: { instanceId: myPmId, message: yield handlers[topic](data) },
        };
        process.send(response);
    }
}));
/**
 * Connects to pm2.
 */
exports.connect = function connect() {
    return new Promise((approve, reject) => pm2.connect(false, (err) => {
        if (err)
            return reject(err);
        pm2.launchBus((err, bus) => {
            if (err)
                return reject(err);
            myBus = bus;
            approve();
        });
    }));
};
/**
 * Determines whether there is a connection with pm2.
 */
exports.isConnected = function isConnected() {
    return !!myBus;
};
/**
 * Disconnects from pm2.
 */
exports.disconnect = function disconnect() {
    return new Promise((approve, reject) => 
    // @ts-ignore: Pass cb to pm2.disconnect
    pm2.disconnect((err) => {
        if (err)
            return reject(err);
        myBus = null;
        approve();
    }));
};
/**
 * Requests messages from processes managed by pm2.
 */
exports.getMessages = function getMessages(topic, data, { filter = (process) => process.name === myName, includeSelfIfUnmanaged = false, timeout = 1000, } = {}) {
    return new Promise((resolve, reject) => {
        assert.ok(exports.isConnected, 'not connected');
        const timer = setTimeout(() => reject(new Error(`${topic} timed out`)), timeout);
        const done = function done(err, messages) {
            clearTimeout(timer);
            if (err)
                reject(err);
            else
                resolve(messages);
        };
        new Promise((resolve, reject) => {
            pm2.list((err, processes) => {
                if (err)
                    return reject(err);
                const messages = [];
                const resolvers = [];
                const resolverBusTargets = [];
                const resolverSelf = () => __awaiter(this, void 0, void 0, function* () { messages.push(yield handlers[topic](data)); });
                if (includeSelfIfUnmanaged && Number.isNaN(myPmId))
                    resolvers.push(resolverSelf());
                for (const process of processes) {
                    if (filter(process)) {
                        if (process.pm_id === myPmId) {
                            resolvers.push(resolverSelf());
                        }
                        else if (typeof process.pm_id === 'number') {
                            resolverBusTargets.push(process.pm_id);
                        }
                    }
                }
                if (resolverBusTargets.length) {
                    resolvers.push(new Promise((resolve, reject) => {
                        const pendingBusTargets = new Set(resolverBusTargets);
                        myBus.on(`process:${myPmId}`, ({ data: { instanceId, message } }) => {
                            if (pendingBusTargets.delete(instanceId)) {
                                messages.push(message);
                                if (!pendingBusTargets.size)
                                    resolve();
                            }
                        });
                        const request = { topic, data: { targetInstanceId: myPmId, data } };
                        resolverBusTargets.forEach((pmId) => {
                            pm2.sendDataToProcessId(pmId, request, (err) => err && reject(err));
                        });
                    }));
                }
                Promise.all(resolvers)
                    .then(() => resolve(messages))
                    .catch(reject);
            });
        })
            .then((messages) => done(null, messages))
            .catch((err) => done(err, []));
    });
};
/**
 * Attaches a handler for the given topic.
 */
exports.onMessage = function onMessage(topic, handler) {
    handlers[topic] = handler;
};
exports.default = { connect: exports.connect, isConnected: exports.isConnected, disconnect: exports.disconnect, getMessages: exports.getMessages, onMessage: exports.onMessage };
//# sourceMappingURL=index.js.map